%******************************************************************************
% TITLE
%   Geophysical Fluid Dynamics - Computer Exercises -
%
%   Diffusion of particles:
%   Diffusion of numerous particles by Brownian motion
%
% HISTORY
%   1999/04/24 Manabu Kanao
%   1999/05/11 Shigeo Yoden
%   2004/05/29 Koichi Shin (RISH) (convert to MATLAB code)
%
% INPUT
%   a0,      % magnitude of static external force
%   a1,      % magnitude of linear external force (Rayleigh friction coefficient)
%   b,       % magnitude of stochastic external force
%   iseed,   % seed for pseudo-random number (RANDLIB): iseed ~= 0
%   nmax,    % number of particles
%   istep,   % interval of graph output
%******************************************************************************
function brown2(aa0,aa1,bb,iseed, nnmax, iistep)

warning off

global a0 a1 b
a0 = aa0;
a1 = aa1;
b = bb;

global nmax itmax istep
nmax = nnmax;
istep = iistep;

rand('state',iseed);
randn('state',iseed);

% nmax: number of particle, itmax: number of iteration
%nmax=5000;
itmax=200;

global dt jbox
dt=0.1;               % dt: temporal grid size
jbox=40;              % jbox: number of bins in histogram

% field of window
xmin=-5.0;xmax=5.0;ymin=-5.0;ymax=5.0;
delx = (xmax-xmin)/jbox;
dely = (ymax-ymin)/jbox;

pdfx = zeros(jbox,itmax+1);
pdfy = zeros(jbox,itmax+1);


%***************************
% set initial values
x = zeros(itmax+1,nmax);
y = zeros(itmax+1,nmax);
%[x,y] = initia(x, y, iseed);


% set initial vecity to zero
uv = zeros(2,nmax);
srand = zeros(2,nmax);

% distribution function
l = 0;
pdfx(:,l+1) = hist(x(l+1,:),xmin:delx:(xmax-delx))'*100/nmax; 
pdfy(:,l+1) = hist(y(l+1,:),ymin:dely:(ymax-dely))'*100/nmax; 
handles = graph(x, y, pdfx, pdfy, dt, l);


srand = zeros(2,nmax);
%***************************
% time evolution
for l=1:itmax
  ud = get(gcf, 'UserData');
  if strcmp(ud, 'stop') || strcmp(ud, 'exit')
    return;
  end

  %srand = randn(2,nmax);
  for n = 1:nmax
    srand(:,n) = rand_nr';
  end
  
  duv = -a1*uv + b*srand;
  duv(2,:) = duv(2,:)-a0; % y direction
  
  duvdt = duv*dt;
    
  uv = uv+duvdt;

  x(l+1,:) = x(l,:) + duvdt(1,:);
  y(l+1,:) = y(l,:) + duvdt(2,:);

  % distribution function
  pdfx(:,l+1) = hist(x(l+1,:),xmin:delx:(xmax-delx))'*100/nmax; 
  pdfy(:,l+1) = hist(y(l+1,:),ymin:dely:(ymax-dely))'*100/nmax; 

  drawnow;
  
  % graph
  graph(x, y, pdfx, pdfy, dt, l);
end


return


%******************************************************************************
% set initial values
% As the initial state, particels are uniformly and isotropically distributed 
% in the circle with the radius r by using random numbers
function [x, y] = initia(x, y, iseed)

global nmax itmax

r = 0.2;
for n=1:nmax
  px = r*(2.0*rand(1) - 1.0);
  py = r*(2.0*rand(1) - 1.0);
  dd = px^2 + py^2;

  while dd > r^2
    px = r*(2.0*rand(1) - 1.0);
    py = r*(2.0*rand(1) - 1.0);
    dd = px^2 + py^2;
  end

  x(0+1,n) = px;
  y(0+1,n) = py;
end

return


%******************************************************************************
% routine for graphics 
function handles = graph(x, y, pdfx, pdfy, dt, l)

  global nmax itmax istep
  global jbox
  global a0 a1 b dt

  persistent hfig
 
  if l == 0
    % initialization
    hfig = gcf;
    set(hfig,'DoubleBuffer','on')
    set(hfig,'BackingStore','off')
  end
  handles.fig = hfig;
  
  if mod(l,istep) ~= 0
    return
  end
  
  % field of window
  xmin=-5.0;xmax=5.0;
  ymin=-5.0;ymax=5.0;

  pmax=15.0;

  %
  fpos = get(gcf,'Position');
  fwhratio = fpos(3)/fpos(4);

  % Erase axes in the figure,
  % unless the UserData exists (for GUI)
  h = get(gcf,'Children');
  for ll = 1:length(h)
    if strcmp(get(h(ll),'Type') ,'axes')
      if isempty(get(h(ll),'UserData'))
        delete(h(ll))
      end
    end
  end

  %
  mainaxes = gca;
  set(gcf, 'CurrentAxes', mainaxes); %axes(mainaxes)
  cla
  set(mainaxes,'Visible','off')
  pos = get(mainaxes,'Position');
  
  x0 = pos(1);
  y0 = pos(2);

  r = min([pos(3),pos(4)]);
  ww = r/fwhratio;
  hh = r;
  ax1 = axes('position',[x0, y0, ww, hh]);
  axx = axes('position',[x0, y0+hh+0.03, ww, 0.3*hh]);
  axy = axes('position',[x0+ww+0.03, y0, 0.3*ww, hh]);

  delx = (xmax-xmin)/jbox;
  dely = (ymax-ymin)/jbox;
  xx = xmin:delx:xmax-delx;
  yy = ymin:dely:ymax-dely;

  %%
%   if a1 == 0
    dx = (b*1.0)*dt;
%   else
%     dx = sqrt((1.0^2)*exp(-2*a1*l*dt) + (b^2)/(2*a1)*(1-exp(-2*a1*l*dt)))*dt;
%   end
  D=(dx^2)/(2*dt);
  t = l*dt;
  nnx = exp(-(xx.^2)/(4*D*t)) / (2*sqrt(pi*D*t)) *delx*100;
  nny = exp(-(yy.^2)/(4*D*t)) / (2*sqrt(pi*D*t)) *dely*100;
  
  %% particles
  set(gcf, 'CurrentAxes', ax1); %axes(ax1);
  plot(x(l+1,:), y(l+1,:),'b.','MarkerSize',4);
  %axis equal;
  axis([xmin, xmax, ymin, ymax])
  xlabel('X')
  ylabel('Y')
  
  %% distribution function (y)
  set(gcf, 'CurrentAxes', axy); %axes(axy)
  [stx,sty] = stairs(xx,pdfy(:,l+1));
  plot(sty,stx-dely/2,'b-')

  axis([0.0 , pmax, ymin, ymax])
  xlabel('PDF(%)')
  set(axy,'YTickLabel','')

  hold on
  plot(nny,yy-a0*dt*l,'k-');
  hold off

  %% distribution function (x)
  set(gcf, 'CurrentAxes', axx); %axes(axx)
  [stx,sty] = stairs(yy,pdfx(:,l+1));
  plot(stx-delx/2,sty,'b-')
 
  axis([xmin, xmax, 0.0 , pmax])
  ylabel('PDF(%)')
  set(axx,'XTickLabel','')

  
  %
  hold on
  plot(xx,nnx,'k-');
  hold off

  
  
  %% title 
  title(sprintf('T= %4.1f',l*dt));

  %
  drawnow

  %
  set(gcf, 'CurrentAxes', mainaxes); %axes(mainaxes);

return
%*****************************************************************************

