%******************************************************************************
% TITLE
%   Geophysical Fluid Dynamics - Computer Exercises -
%
%   Fluid parcel in steady flow from Lagrange view (1)
%   Strain
%
% HISTORY
%   1999/04/24 Tomonori Ohmiya
%   1999/05/11 Shigeo Yoden
%   2004/05/24 Koichi Shin (RISH) (convert to MATLAB code)
%
% INPUT
%      u0, dudx, dudy,  % u-component of steady flow: u = u0 + dudx*x + dudy*y
%      v0, dvdx, dvdy,  % v-component of steady flow: v = v0 + dvdx*x + dvdy*y
%      dt
%      itmax
%******************************************************************************
function deform1(uu0,ddudx,ddudy, vv0,ddvdx,ddvdy, varargin)

global u0 dudx dudy v0 dvdx dvdy;
global xmin xmax ymin ymax nx ny dt;

u0= uu0;
dudx = ddudx;
dudy = ddudy;
v0 = vv0;
dvdx = ddvdx;
dvdy = ddvdy;

% nmax: number of particle
nmax=360;
% itmax: number of iteration 
itmax=30;
% dt: temporal grid size(=tend-t)
dt=1.0;
% istep: number of time step
istep=10;

xmin=-200.;
xmax=200.;
ymin=-100.;
ymax=100.;
nx=21;
ny=11;

%**********
% Argument processing
for k = 1:length(varargin)
  if isempty(varargin{k})
    continue
  end
  switch k 
   case 1
    dt = varargin{1};
   case 2
    itmax = varargin{2};
  end
end

work = zeros(2,3);

x = zeros(nmax+1,itmax+1);
y = zeros(nmax+1,itmax+1);

u = zeros(nx,ny);
v = zeros(nx,ny);


%***************************
% flow field (u,v) 
delx = (xmax-xmin)/(nx-1);
dely = (ymax-ymin)/(ny-1);
for jj=1:ny
  for ii=1:nx
    xx = xmin + delx*(ii-1);
    yy = ymin + dely*(jj-1);

    u(ii,jj) = u0 + dudx*xx + dudy*yy;
    v(ii,jj) = v0 + dvdx*xx + dvdy*yy;
  end
end

%rmiss = 0.0;
%u(1,:)=rmiss;
%v(1,:)=rmiss;
%u(nx,:)=rmiss;
%v(nx,:)=rmiss;
%u(:,1)=rmiss;
%v(:,1)=rmiss;
%u(:,ny)=rmiss;
%v(:,ny)=rmiss;

%***************************
% initial condition 
t = 0.0;
theta = 2.0*pi/nmax;       

n=0:nmax;
x(:,1) = 40.0*cos(n*theta)';
y(:,1) = 40.0*sin(n*theta)';

if u0 > 0.0
  x(:,1) = x(:,1) -150.0;
end

%***************************
% time-integration

xy = zeros(2,nmax+1,itmax+1);
xy(1,:,:) = x;
xy(2,:,:) = y;

h = graphics_init(u,v,xy);

tend = t + dt;
for ii=(2:itmax)
  ud = get(gcf, 'UserData');
  if strcmp(ud, 'stop') || strcmp(ud, 'exit')
    return;
  end
  
  xy(:,:,ii) = odrkdu(2,'odrkg','fcn1', t, tend, istep, xy(:,:,ii-1));

  h = graphics(xy,ii,h);
end  

return


%***************************
% graph
function h = graphics_init(u,v,xy)

global xmax xmin ymax ymin nx ny dt

[xx,yy] = meshgrid(xmin:(xmax-xmin)/(nx-1):xmax,ymin:(ymax-ymin)/(ny-1):ymax);
quiver(xx,yy,u',v','c')

axis equal
axis([xmin, xmax, ymin, ymax])
xlabel('X')
ylabel('Y')

hold on
h = plot(xy(1,:,1), xy(2,:,1),'r.');
hold off

title(sprintf('T= %4.1f',0));
return

%%%%%%
function h = graphics(xy,n,h)

global xmax xmin ymax ymin nx ny dt

set(h,'LineStyle','-','Marker','none','Color',[1 0.5 0.5]);

hold on
h = plot(xy(1,:,n), xy(2,:,n),'r.');
hold off

title(sprintf('T= %4.1f',n*dt));

drawnow
return
%******************************************************************************
