%******************************************************************************
% TITLE
%   Geophysical Fluid Dynamics - Computer Exercises -
%
%   Fluid parcel in steady flow from Lagrange view (2)
%   Rotation
%
% HISTORY
%   1999/04/24 Tomonori Ohmiya
%   1999/05/11 Shigeo Yoden
%   2004/05/24 Koichi Shin (RISH) (convert to MATLAB code)
%
% INPUT
%     omega,    % angular velocity of rigid rotation: v_tan = r*omega
%     redge,    % radius of the region of rigid rotation
%     xoffst,   % initial position of square (x)
%     yoffst,   % initial position of square (y)
%     dt
%     itmax
%******************************************************************************
function deform2(oomega, rredge, xoffst, yoffst, varargin)

global xmin xmax ymin ymax nx ny dt
global omega redge;
omega = oomega;
redge = rredge;

%nmax=480; % nmax: number of particle
nmax=320;
%itmax=30; % itmax: number of iteration

%dt=1.0; % dt: temporal grid size(=tend-t)
istep=10; % istep: number of time step

%**********
% Argument processing
for k = 1:length(varargin)
  if isempty(varargin{k})
    continue
  end
  switch k 
   case 1
    dt = varargin{1};
   case 2
    itmax = varargin{2};
  end
end


%**********
xmin=-100.;
xmax=100.;
ymin=-100.;
ymax=100.;
nx=31;
ny=31;

% ----------
work = zeros(2,3);
x = zeros(nmax+1,itmax+1);
y = zeros(nmax+1,itmax+1);
u = zeros(nx,ny);
v = zeros(nx,ny);


%***************************
% flow field (u,v)
delx = (xmax-xmin)/(nx-1);
dely = (ymax-ymin)/(ny-1);
for j=1:ny
  for i=1:nx
    xx = xmin + delx*(i-1);
    yy = ymin + dely*(j-1);
    rr = sqrt( xx^2 + yy^2 );

    if rr <= redge
      vtan = rr*omega;
    else
      vtan = redge^2 *omega/rr;
    end
        
    if yy >= 0.0
      theta =   acos(xx/rr);
    else  
      theta = - acos(xx/rr);
    end  
    
    u(i,j) = - vtan*sin(theta);
    v(i,j) =   vtan*cos(theta);
  end
end

%***************************
% initial condition
t = 0.0;
hr = 30.;

for n=1:(nmax/4)
  dx = 2*hr/(nmax/4);
  x(n         +1,1) = - hr + dx*(n-1) + xoffst;
  x(n+  nmax/4+1,1) =   hr            + xoffst;
  x(n+2*nmax/4+1,1) =   hr - dx*(n-1) + xoffst;
  x(n+3*nmax/4+1,1) = - hr            + xoffst;
  y(n         +1,1) = - hr            + yoffst;
  y(n+  nmax/4+1,1) = - hr + dx*(n-1) + yoffst;
  y(n+2*nmax/4+1,1) =   hr            + yoffst;
  y(n+3*nmax/4+1,1) =   hr - dx*(n-1) + yoffst;
end

x(1,1) = x(nmax+1,1);
y(1,1) = y(nmax+1,1);


%***************************
% time-integration

xy = zeros(2,nmax+1,itmax+1);
xy(1,:,:) = x;
xy(2,:,:) = y;

h = graphics_init(xy,u,v,1);

tend = t + dt;
for ii=2:(itmax+1)
  ud = get(gcf, 'UserData');
  if strcmp(ud, 'stop') || strcmp(ud, 'exit')
    return;
  end
  
  xy(:,:,ii) = odrkdu(2,'odrkg','fcn2', t, tend, istep, xy(:,:,ii-1));

  h = graphics(xy,ii,h);
end  

return


%***************************
% graph
function h = graphics_init(xy,u,v,n)

global xmin xmax ymin ymax nx ny dt

h.fig = gcf;
set(h.fig,'KeyPressFcn','pauseplot(gcbo)')

[xx,yy] = meshgrid(xmin:(xmax-xmin)/(nx-1):xmax,ymin:(ymax-ymin)/(ny-1):ymax);
quiver(xx,yy,u',v','c')

axis equal
axis([xmin, xmax, ymin, ymax])
xlabel('X');
ylabel('Y');

%
hold on
h.plot = plot(xy(1,:,n), xy(2,:,n),'r.');
hold off

h.title = title(sprintf('T= %4.1f', (n-1)*dt));

drawnow

return
%*****************************************************************************

function h = graphics(xy,n,h)

global xmin xmax ymin ymax nx ny dt


set(h.plot,'LineStyle','-','Marker','none','Color', [1 0.5 0.5]);
hold on
h.plot = plot(xy(1,:,n), xy(2,:,n),'r.');
hold off
set(h.title,'String',sprintf('T= %4.1f', (n-1)*dt))

drawnow
return;

