%******************************************************************************
% TITLE
%   Geophysical Fluid Dynamics - Computer Exercises -
%
%   1-D Linear Advection and Diffusion
%   Diffusion equation on a moving coordinate system
%
% HISTORY
%   1999/05/10 Takahiro Koyama
%   1999/05/12 Shigeo Yoden
%   2004/05/24 Koichi Shin (RISH) (MATLAB code)
%   2005/12/09 Katsushi Sato (alter GUI)
%
% MEMO
%   The solution of diffution equation is translated in order to express
%   the advection instead of using the analytic solution for advection
%   equation, since it saves number of grid and calculation time and gives
%   nicer graph. 
%
% USAGE
%    linadv(ijk,amp,rc,rnu,nmax,tstep)
%    The arguments nmax and tstep are optional.
%
% INPUT
%    ijk,     % initial shape (1: sine curve, 2: step, 3: triangle)
%    amp,     % initial amplitude (maximum value)
%    rc,      % advection velocity
%    rnu,     % diffusion coefficient
%    nmax     % maximum number of time step
%    tstep    % pause time at each step (sec)
%******************************************************************************
function linadv(ijk,amp,rc,rnu, varargin)

global xmin xmax umin umax dt tstep

%**********
% Arguments processing
%   If no nmax and tstep, default values are used.
switch length(varargin)
 case 1
  nmax = varargin{1};
  tstep = 1;
 case 2
  nmax = varargin{1};
  tstep = varargin{2};
 otherwise
  nmax = 30;
  tstep = 1;
end
  
%**********
% Initialization
imax=200;  % (grid number)/2
imin=-imax;

xmin= 0.0;  xmax=200.0;
umin=-0.1;  umax=  5.0;

dt=1.0;                 % temporal grid size
dx=(xmax-xmin)/imax;    % spatial grid size

%************
% Set initial values
x = zeros(imax*2+1,nmax+1);
u = zeros(imax*2+1,nmax+1);

switch ijk
 case 1  % sine curve
  ii=0:(imax/2);
  u(ii+imax+1,0+1) = (0.5.*amp.*(-cos(2.0*pi.*ii./(imax/2)) + 1))';
 case 2  % step function
  ii=(imax/8+1):(imax*3/8);
  u(ii+imax+1,0+1) = amp;
 case 3  % triangular function
  ii=(imax/8+1):(imax/4);
  u(ii+imax+1,0+1) =       (8.*amp.*(ii-imax/8)./imax)';
  ii=(imax/4+1):(imax*3/8);
  u(ii+imax+1,0+1) = (amp - 8.*amp.*(ii-imax/4)./imax)';
 otherwise
  disp('ijk should be 1, 2 or 3');
  return;
end
      
x(:,0+1) = (imin:imax)'*dx;


%**************
% Main loop
%   diffution equation on a moving coordinate system
h = graphics_init(x,u); % initialize graphics

for n=1:nmax
  ud = get(gcf, 'UserData');
  if strcmp(ud, 'stop') || strcmp(ud, 'exit')
    return;
  end
  
  t = n*dt;                % time
  x(:,n+1) = x(:,1)+rc*t;  % shift x by rc*t
    
  if rnu ~= 0
    for ii=imin:imax
      jj=0:(imax/2);    % NOTE: Range of integration depends on the range of x with non-zero values initially.
      xj   = jj*dx;
      rint = u(jj+imax+1,0+1).* exp( -(x(ii+imax+1,0+1)-xj').^2 / (4*rnu*t) ) *dx;
      rint = sum(rint,1);
      u(ii+imax+1,n+1) = rint / sqrt(4*pi*rnu*t);
    end
  else
    % no diffusion
    u(:,n+1) = u(:,1);
  end

  if mod(n,tstep) == 0
    h = graphics(x,u,n,h);  % graphics
  end
end


return;


%******************************************************************************
% Initialize graphics
%
% INPUT
%   x, u 
% OUTPUT
%   h: handle of plot (to thin the previous plot in drawing the next step)
%
function h = graphics_init(x,u)
global xmin xmax umin umax dt


set(gcf,'doublebuffer','on'); % to avoid blinking
set(gca,'nextplot','replacechildren'); % for GUI
delete(get(gca,'title')); % to avoid miss-positioning in GUI
delete(get(gca,'xlabel'));
delete(get(gca,'ylabel'));


% plot initial values
h = plot(x(:,1), u(:,1),'r-','LineWidth',2);

axis([xmin, xmax, umin, umax]); % set field of drawing

xlabel('X');
ylabel('U');
title(sprintf('Linear Advection and Diffusion: T=%4.1f',0));

return


%******************************************************************************
% Graphics
%
% INPUT
%   x, u 
%   n: present step number
%   h: handle of previous plot
% OUTPUT
%   h: handle of plot
%
function h = graphics(x,u,n,h)
global xmin xmax umin umax dt

% change the color and width of previous step
set(h,'Color',[1 0.5 0.5],'LineWidth',1);

% draw line and title, using "hold" to keep the previous lines
hold on
h = plot(x(:,n+1), u(:,n+1),'r-','LineWidth',2);
hold off

title(sprintf('Linear Advection and Diffusion: T=%4.1f',n*dt));

drawnow
return;

