%******************************************************************************
% TITLE
%   Geophysical Fluid Dynamics - Computer Exercises -
%
%   1-D Advection and Diffusion
%   Burgers equation
%
% HISTORY
%   1999/05/06 Seiya Nishizawa
%   1999/05/12 Shigeo Yoden
%   2004/05/24 Koichi Shin (RISH) (MATLAB code)
%   2005/12/09 Katsushi Sato (alter GUI)
%
% USAGE
%    nladv(ijk,amp,rnu,nstep,dt,twait)
%    The arguments nstep and twait are optional.
%
% INPUT
%    ijk,     % initial shape (1: sine curve, 2: step, 3: triangle)
%    amp,     % initial amplitude (maximum value)
%    rnu,     % diffusion coefficient
%    nstep,   % maximum number of time step
%    dt,      % temporal grid size
%    twait    % pause time at each step (sec)
%******************************************************************************
function nladv(ijk,amp,rrnu, varargin)

global xmin xmax umin umax twait dt % for display
global dx rnu; % for external functions
rnu = rrnu;

%********
% default values
nstep = 30;
dt = 1.0; % dt: temporal grid size (=tend-t)
twait = 0.1;

%**********
% Arguments processing
for k = 1:length(varargin)  
  switch k 
   case 1
    if ~isempty(varargin{1})
      nstep = varargin{1};
    end
   case 2
    if ~isempty(varargin{2})
      dt = varargin{2};
    end
   case 3
    if ~isempty(varargin{3})
      twait = varargin{3};
    end
  end
end

%*******
% imax: number of grid
imax=200;

% istep: number of time step
istep=10;
if dt > 1
  istep = floor(dt)*10;
end

% display field
xmin=0.;xmax=200.;
umin=-0.1;umax=5.;

dx = (xmax-xmin)/imax;  % dx: spatial grid size

work = zeros(imax+1,3);

%******************************
% initial values
u = zeros(imax+1,nstep+1);
x = zeros(imax+1,nstep+1);

switch ijk
 case 1  % sine curve
  ii=0:(imax/2);
  u(ii+1,0+1) = (0.5.*amp.*(-cos(2.0*pi.*ii./(imax/2)) + 1))';
 case 2  % step function
  ii=(imax/8+1):(imax*3/8);
  u(ii+1,0+1) = amp;
 case 3  % triangular function
  ii=(imax/8+1):(imax/4);
  u(ii+1,0+1) =      (8.*amp.*(ii-imax/8)./imax)';
  ii=(imax/4+1):(imax*3/8);
  u(ii+1,0+1) = (amp -8.*amp.*(ii-imax/4)./imax)';
 otherwise
  % warning
  disp('ijk should be 1, 2 or 3');
  return;
end

u(imax+1,0+1) = u(0+1,0+1);

uw = u(:,0+1); % set uw to initial values
x = (0:imax)'*dx;

%******************************
% time integration

h = graphics_init(x,u); % initialize graphics

t=0.0;
for n=1:nstep
  ud = get(gcf, 'UserData');
  if strcmp(ud, 'stop') || strcmp(ud, 'exit')
    return;
  end

  tend = t + dt;

  % integrate with constant grid size
  uw = odrkdu(imax+1, 'odrkg', 'fcn2', t, tend, istep, uw, work);
  t = tend;
  
  u(:,n+1) = uw;

  h = graphics(x,u,n,h); % graphics
end



%******************************
% graphics
function h = graphics_init(x,u)

global xmin xmax umin umax twait dt

set(gcf,'doublebuffer','on'); % avoid blinking
%set(gca,'nextplot','replacechildren'); % for GUI

title(sprintf('Nonlinear Advection and Diffusion: T= %4.1f',0));

h = plot(x, u(:,1),'r-','LineWidth',2);

axis([xmin, xmax, umin, umax])
xlabel('X')
ylabel('U')



return;


function h = graphics(x,u,n,h)

global xmin xmax umin umax twait dt

set(h,'Color',[1 0.5 0.5],'LineWidth',1);

title(sprintf('Nonlinear Advection and Diffusion: T= %4.1f',n*dt));

hold on
h = plot(x, u(:,n+1),'r-','LineWidth',2);
hold off


pause(twait);
return


%**********************************************************************
